@@ -27,8 +27,6 @@ import json
 
 class Network(object):
   '''
-  version 1.13.5
-
   Clustergrammer.py takes a matrix as input (either from a file of a Pandas DataFrame), normalizes/filters, hierarchically clusters, and produces the :ref:`visualization_json` for :ref:`clustergrammer_js`.
 
   Networks have two states:
@@ -41,6 +39,8 @@ class Network(object):
   '''
 
   def __init__(self, widget=None):
+
+    # print('INITIALIZING')
     initialize_net.main(self, widget)
 
   def reset(self):
@@ -93,80 +93,59 @@ class Network(object):
                  linkage_type='average', sim_mat=False, filter_sim=0.0,
                  calc_cat_pval=False, run_enrichr=None, enrichrgram=None):
     '''
-    The main function performs hierarchical clustering, optionally generates filtered views (e.g. row-filtered views), and generates the :``visualization_json``.
+    The main function performs hierarchical clustering, optionally generates
+    filtered views (e.g. row-filtered views), and generates the :
+    ``visualization_json``.
 
     Used to set views equal to ['N_row_sum', 'N_row_var']
     '''
     initialize_net.viz(self)
 
-    make_clust_fun.make_clust(self, dist_type=dist_type, run_clustering=run_clustering,
-                                   dendro=dendro,
-                                   requested_views=views,
-                                   linkage_type=linkage_type,
-                                   sim_mat=sim_mat,
-                                   filter_sim=filter_sim,
-                                   calc_cat_pval=calc_cat_pval,
-                                   run_enrichr=run_enrichr,
-                                   enrichrgram=enrichrgram)
-
-  def make_clust(self, dist_type='cosine', run_clustering=True,
-                 dendro=True, views=['N_row_sum', 'N_row_var'],
-                 linkage_type='average', sim_mat=False, filter_sim=0.0,
-                 calc_cat_pval=False, run_enrichr=None, enrichrgram=None):
-    '''
-    ... Will be deprecated, renaming method cluster ...
-    The main function performs hierarchical clustering, optionally generates filtered views (e.g. row-filtered views), and generates the :``visualization_json``.
-    '''
-    print('make_clust method will be deprecated in next version, please use cluster method.')
-    initialize_net.viz(self)
-
-    make_clust_fun.make_clust(self, dist_type=dist_type, run_clustering=run_clustering,
-                                   dendro=dendro,
-                                   requested_views=views,
-                                   linkage_type=linkage_type,
-                                   sim_mat=sim_mat,
-                                   filter_sim=filter_sim,
-                                   calc_cat_pval=calc_cat_pval,
-                                   run_enrichr=run_enrichr,
-                                   enrichrgram=enrichrgram)
-
-  def produce_view(self, requested_view=None):
-    '''
-    This function is under development and will produce a single view on demand.
-    '''
-    print('\tproduce a single view of a matrix, will be used for get requests')
-
-    if requested_view != None:
-      print('requested_view')
-      print(requested_view)
+    make_clust_fun.make_clust(self, dist_type=dist_type,
+                                    run_clustering=run_clustering,
+                                    dendro=dendro,
+                                    requested_views=views,
+                                    linkage_type=linkage_type,
+                                    sim_mat=sim_mat,
+                                    filter_sim=filter_sim,
+                                    calc_cat_pval=calc_cat_pval,
+                                    run_enrichr=run_enrichr,
+                                    enrichrgram=enrichrgram)
 
   def swap_nan_for_zero(self):
     '''
     Swaps all NaN (numpy NaN) instances for zero.
     '''
-    # # may re-instate this in some form
-    # self.dat['mat_orig'] = deepcopy(self.dat['mat'])
-
     self.dat['mat'][np.isnan(self.dat['mat'])] = 0
 
-  def load_df(self, df):
+  def load_df(self, df_ini, meta_col=None, meta_row=None):
     '''
     Load Pandas DataFrame.
     '''
-    # self.__init__()
     self.reset()
 
-    df_dict = {}
-    df_dict['mat'] = deepcopy(df)
-    # always define category colors if applicable when loading a df
-    data_formats.df_to_dat(self, df_dict, define_cat_colors=True)
+    # load dataframe
+    df = deepcopy(df_ini)
+
+    self.meta_cat = False
+
+    # load metadata
+    if isinstance(meta_col, pd.DataFrame):
+      self.meta_col = meta_col
+      self.meta_cat = True
+
+    if isinstance(meta_row, pd.DataFrame):
+      self.meta_row = meta_row
+      self.meta_cat = True
+
+    data_formats.df_to_dat(self, df, define_cat_colors=True)
 
   def export_df(self):
     '''
     Export Pandas DataFrame/
     '''
-    df_dict = data_formats.dat_to_df(self)
-    return df_dict['mat']
+    df = data_formats.dat_to_df(self)
+    return df
 
   def df_to_dat(self, df, define_cat_colors=False):
     '''
@@ -223,27 +202,21 @@ class Network(object):
     method passes the visualization JSON to the instantiated widget, which is
     returned and visualized on the front-end.
     '''
+    # run clustering if necessary
+    if len(self.viz['row_nodes']) == 0:
+      self.cluster()
 
-    if hasattr(self, 'widget_class') == True:
+      # add manual_category to viz json
+      if 'manual_category' in self.dat:
+        self.viz['manual_category'] = self.dat['manual_category']
 
-      # run clustering if necessary
-      if len(self.viz['row_nodes']) == 0:
-        self.cluster()
+      # add pre-z-score data to viz
+      if 'pre_zscore' in self.dat:
+        self.viz['pre_zscore'] = self.dat['pre_zscore']
 
-        # add manual_category to viz json
-        if 'manual_category' in self.dat:
-          self.viz['manual_category'] = self.dat['manual_category']
+    self.widget_instance = self.widget_class(network = self.export_viz_to_widget(which_viz))
 
-        # add pre-z-score data to viz
-        if 'pre_zscore' in self.dat:
-          self.viz['pre_zscore'] = self.dat['pre_zscore']
-
-      self.widget_instance = self.widget_class(network = self.export_viz_to_widget(which_viz))
-
-      return self.widget_instance
-    else:
-      print('Can not make widget because Network has no attribute widget_class')
-      print('Please instantiate Network with clustergrammer_widget using: Network(clustergrammer_widget)')
+    return self.widget_instance
 
 
   def widget_df(self):
@@ -369,11 +342,11 @@ class Network(object):
     df = df.clip(lower=lower, upper=upper)
     self.load_df(df)
 
-  def normalize(self, df=None, norm_type='zscore', axis='row', keep_orig=False):
+  def normalize(self, df=None, norm_type='zscore', axis='row'):
     '''
     Normalize the matrix rows or columns using Z-score (zscore) or Quantile Normalization (qn). Users can optionally pass in a DataFrame to be normalized (and this will be incorporated into the Network object).
     '''
-    normalize_fun.run_norm(self, df, norm_type, axis, keep_orig)
+    normalize_fun.run_norm(self, df, norm_type, axis)
 
   def downsample(self, df=None, ds_type='kmeans', axis='row', num_samples=100, random_state=1000):
     '''
@@ -432,14 +405,6 @@ class Network(object):
     for inst_data in cat_data:
       categories.add_cats(self, axis, inst_data)
 
-  def dendro_cats(self, axis, dendro_level):
-    '''
-    Generate categories from dendrogram groups/clusters. The dendrogram has 11
-    levels to choose from 0 -> 10. Dendro_level can be given as an integer or
-    string.
-    '''
-    categories.dendro_cats(self, axis, dendro_level)
-
   def Iframe_web_app(self, filename=None, width=1000, height=800):
 
     link = iframe_web_app.main(self, filename, width, height)
@@ -1150,22 +1115,38 @@ class Network(object):
   def get_manual_category(self, axis, cat_title):
     try:
       export_dict = {}
+
+      # Category Names
+      #######################
       try:
-        # print(json.loads(self.widget_instance.custom_cat))
         export_dict[cat_title] = pd.Series(json.loads(self.widget_instance.custom_cat)[axis][cat_title])
+
+        if hasattr(self, 'meta_cat') == True:
+
+          if axis == 'row':
+            print('saving to self')
+            self.meta_row[cat_title] = export_dict[cat_title]
+
+          elif axis == 'col':
+            print('saving to self')
+            self.meta_col[cat_title] = export_dict[cat_title]
+
       except:
         print('unable to load category, please check title')
-      ini_new_colors = json.loads(self.widget_instance.custom_cat)[axis + '_cat_colors']
 
+      # Category Colors
+      #######################
+      ini_new_colors = json.loads(self.widget_instance.custom_cat)[axis + '_cat_colors']
       # drop title from category colors
-      export_dict['new_colors'] = {}
+      export_dict['cat_colors'] = {}
       for inst_cat in ini_new_colors:
         if (': ' in inst_cat):
-          export_dict['new_colors'][inst_cat.split(': ')[1]] = ini_new_colors[inst_cat]
+          export_dict['cat_colors'][inst_cat.split(': ')[1]] = ini_new_colors[inst_cat]
         else:
-          export_dict['new_colors'][inst_cat] = ini_new_colors[inst_cat]
+          export_dict['cat_colors'][inst_cat] = ini_new_colors[inst_cat]
 
-      return export_dict
+      if hasattr(self, 'meta_cat') == False:
+        return export_dict
     except:
         print('please set custom category')
 
@@ -1175,4 +1156,31 @@ class Network(object):
       # umi norm
       barcode_umi_sum = df.sum()
       df_umi = df.div(barcode_umi_sum)
-      return df_umi
\ No newline at end of file
+      return df_umi
+
+  @staticmethod
+  def make_df_from_cols(cols):
+    inst_col = cols[0]
+
+    cat_titles = []
+    for inst_info in inst_col[1:]:
+        inst_title = inst_info.split(': ')[0]
+        cat_titles.append(inst_title)
+
+    clean_cols = []
+    for inst_col in cols:
+        inst_clean = []
+        for inst_info in inst_col:
+            if ': ' in inst_info:
+                inst_clean.append(inst_info.split(': ')[1])
+            else:
+                inst_clean.append(inst_info)
+        clean_cols.append(tuple(inst_clean))
+
+    df_ini = pd.DataFrame(data=clean_cols).set_index(0)
+    mat = df_ini.get_values()
+    rows = df_ini.index.tolist()
+
+    df_meta = pd.DataFrame(data=mat, index=rows, columns=cat_titles)
+
+    return df_meta
\ No newline at end of file
